(ns leiningen.new.python
  (:require [leiningen.new.templates :refer [renderer name-to-path ->files
                                             *dir*]]
            [clojure.java.io :as io]
            [clojure.java.shell :as shell]
            [leiningen.core.main :as main]))

(def render (renderer "python"))

(defn sh-output
  "return shell/sh's output, either stdout or stderr"
  [result]
  (let [output (str (:out result)
                    (:err result))]
    (if (not (empty? output))
      (main/info output))))

(defn python
  "a standard python project with virtualenv, githooks, pep8, pylint etc"
  [name]
  (let [data {:name name
              :sanitized (name-to-path name)}
        plain-files ["Makefile"
                     "requirements.txt"
                     "utils/bootstrap"
                     "utils/install-git-hooks"
                     "utils/trigger-jenkins-build"
                     "git-hooks/post-commit"
                     "git-hooks/post-merge"
                     "git-hooks/pre-commit"
                     ".gitignore"]
        post-run-commands [["chmod" "-R" "+x" "utils" "git-hooks/"]]]
    (main/info "Generating new python project:" name)
    (apply
     ->files
     data
     ["{{name}}/__init__.py" ""]
     (map (fn [f] [f (render f data)]) plain-files))
    (shell/with-sh-dir *dir*
      (doseq [cmd post-run-commands]
      (sh-output (apply shell/sh cmd))))
    ;; run git init if no .git found.
    (if (not (.exists (io/file *dir* ".git")))
      (sh-output (shell/sh "git" "init" :dir *dir*)))))
